@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import base64
 import email
 import hashlib
@@ -81,6 +82,10 @@ class Key(object):
         #
         self.local_hashes = {}
         self.user_meta = {}
+        self.tagging_count = None
+        self.object_type = None
+        self.append_next_position = None
+
 
     def __repr__(self):
         if self.bucket:
@@ -215,6 +220,26 @@ class Key(object):
            url = url.replace('https://'+self.bucket.name+'.','https://')
         
         return url
+    
+    # GetObjectPresignedUrl 生成临时上传链接
+    def get_presigned_url(self, expires_in, headers=None, expires_in_absolute=False, version_id=None, domain=False):
+        """
+        生成临时上传链接
+        """
+        version_id = version_id or self.version_id
+        if headers is None:
+            headers = {}
+        else:
+            headers = headers.copy()
+
+        url = self.bucket.connection.generate_url(expires_in, 'PUT',
+                                                   self.bucket.name, self.name,
+                                                   headers=headers, expires_in_absolute=expires_in_absolute, version_id=version_id)
+        if domain is True :
+           url = url.replace('http://'+self.bucket.name+'.','http://')
+           url = url.replace('https://'+self.bucket.name+'.','https://')
+        
+        return url
 
     def should_retry(self, response, chunked_transfer=False):
         provider = self.bucket.connection.provider
@@ -377,7 +402,7 @@ class Key(object):
                 self.name,
                 data=fp,
                 headers=headers,
-                query_args=query_args
+                query_args=query_args,
                 )
             if resp and resp.status > 299:
                 raise provider.storage_response_error(resp.status, resp.reason, resp.read())
@@ -585,7 +610,7 @@ class Key(object):
     def set_contents_from_filename(self, filename, headers=None, replace=True,
                                    cb=None, num_cb=10, policy=None, md5=None,
                                    reduced_redundancy=False,
-                                   encrypt_key=False, calc_encrypt_md5=True):
+                                   encrypt_key=False, calc_encrypt_md5=True, query_args=None):
         """
         Store an object in S3 using the name of the Key object as the
         key in S3 and the contents of the file named by 'filename'.
@@ -596,12 +621,12 @@ class Key(object):
             return self.set_contents_from_file(fp, headers, replace, cb,
                                                num_cb, policy, md5,
                                                reduced_redundancy,
-                                               encrypt_key=encrypt_key, calc_encrypt_md5=calc_encrypt_md5)
+                                               encrypt_key=encrypt_key, calc_encrypt_md5=calc_encrypt_md5,query_args= query_args)
 
     def set_contents_from_string(self, string_data, headers=None, replace=True,
                                  cb=None, num_cb=10, policy=None, md5=None,
                                  reduced_redundancy=False,
-                                 encrypt_key=False, calc_encrypt_md5=True):
+                                 encrypt_key=False, calc_encrypt_md5=True, query_args=None):
         """
         Store an object in S3 using the name of the Key object as the
         key in S3 and the string 's' as the contents.
@@ -613,7 +638,7 @@ class Key(object):
         fp = BytesIO(string_data)
         r = self.set_contents_from_file(fp, headers, replace, cb, num_cb,
                                         policy, md5, reduced_redundancy,
-                                        encrypt_key=encrypt_key, calc_encrypt_md5=calc_encrypt_md5)
+                                        encrypt_key=encrypt_key, calc_encrypt_md5=calc_encrypt_md5, query_args=query_args)
         fp.close()
         return r
 
@@ -832,7 +857,7 @@ class Key(object):
         self.mode = None
         self.closed = True
 
-    def get_contents_to_file(self, fp, headers=None,
+    def get_contents_to_file(self, fp, byte_range=None, headers=None,
                              cb=None, num_cb=10,
                              torrent=False,
                              version_id=None,
@@ -842,7 +867,15 @@ class Key(object):
         Retrieve an object from S3 using the name of the Key object as the
         key in S3.  Write the contents of the object to the file pointed
         to by 'fp'.
+
+        :param byte_range: 指定下载范围。参见: ref:`byte_range`
         """
+
+        range_string = _make_range_string(byte_range)
+        if range_string:
+            if not headers:
+                headers = {'range': range_string}
+
         if self.bucket is not None:
             if res_download_handler:
                 res_download_handler.get_file(self, fp, headers, cb, num_cb,
@@ -853,7 +886,7 @@ class Key(object):
                               version_id=version_id,
                               response_headers=response_headers)
 
-    def get_contents_to_filename(self, filename, headers=None,
+    def get_contents_to_filename(self, filename, byte_range=None, headers=None,
                                  cb=None, num_cb=10,
                                  torrent=False,
                                  version_id=None,
@@ -864,10 +897,18 @@ class Key(object):
         key in S3.  Store contents of the object to a file named by 'filename'.
         See get_contents_to_file method for details about the
         parameters.
+
+        :param byte_range: 指定下载范围。参见: ref:`byte_range`
         """
+
+        range_string = _make_range_string(byte_range)
+        if range_string:
+            if not headers:
+                headers = {'range': range_string}
+
         try:
             with open(filename, 'wb') as fp:
-                self.get_contents_to_file(fp, headers, cb, num_cb,
+                self.get_contents_to_file(fp, byte_range, headers, cb, num_cb,
                                           torrent=torrent,
                                           version_id=version_id,
                                           res_download_handler=res_download_handler,
@@ -881,10 +922,9 @@ class Key(object):
                 modified_tuple = email.utils.parsedate_tz(self.last_modified)
                 modified_stamp = int(email.utils.mktime_tz(modified_tuple))
                 os.utime(fp.name, (modified_stamp, modified_stamp))
-            except Exception:
+            except :
                 pass
-
-    def get_contents_as_string(self, headers=None,
+    def get_contents_as_string(self, byte_range=None, headers=None,
                                cb=None, num_cb=10,
                                torrent=False,
                                version_id=None,
@@ -894,9 +934,12 @@ class Key(object):
         key in S3.  Return the contents of the object as a string.
         See get_contents_to_file method for details about the
         parameters.
+
+        :param byte_range: 指定下载范围。参见: ref:`byte_range`
         """
+
         fp = BytesIO()
-        self.get_contents_to_file(fp, headers, cb, num_cb, torrent=torrent,
+        self.get_contents_to_file(fp, byte_range, headers, cb, num_cb, torrent=torrent,
                                   version_id=version_id,
                                   response_headers=response_headers)
         value = fp.getvalue()
@@ -980,3 +1023,48 @@ class Key(object):
     def compute_encrypted_md5(self, fp):
         hex_digest, b64_digest = compute_encrypted_md5(fp)
         return b64_digest
+
+    def handle_tagging_count(self,response):
+        provider = self.bucket.connection.provider
+        if provider:
+            sc = response.getheader(provider.tagging_count_header)
+            if sc:
+                self.tagging_count = sc
+
+    def handle_object_type(self,response):
+        provider = self.bucket.connection.provider
+        if provider:
+            sc = response.getheader(provider.object_type_header)
+            if sc:
+                self.object_type = sc
+
+    def handle_next_position(self,response):
+        provider = self.bucket.connection.provider
+        if provider:
+            sc = response.getheader(provider.append_next_position_header)
+            if sc:
+                self.append_next_position = sc
+
+
+
+def _make_range_string(range):
+    if range is None:
+        return ''
+
+    start = range[0]
+    last = range[1]
+
+    if start is None and last is None:
+        return ''
+
+    return 'bytes=' + _range(start, last)
+
+
+def _range(start, last):
+    def to_str(pos):
+        if pos is None:
+            return ''
+        else:
+            return str(pos)
+
+    return to_str(start) + '-' + to_str(last)
\ No newline at end of file
