@@ -33,6 +33,9 @@ MFA_HEADER_KEY = 'mfa-header'
 SERVER_SIDE_ENCRYPTION_KEY = 'server-side-encryption-header'
 VERSION_ID_HEADER_KEY = 'version-id-header'
 RESTORE_HEADER_KEY = 'restore-header'
+TAGGING_COUNT_HEADER_KEY = 'tagging-count-header'
+OBJECT_TYPE_HEADER_KEY = 'object-type-header'
+APPEND_NEXT_POSITION_HEADER_KEY = 'append-next-position-header'
 
 STORAGE_COPY_ERROR = 'StorageCopyError'
 STORAGE_CREATE_ERROR = 'StorageCreateError'
@@ -113,6 +116,9 @@ class Provider(object):
             STORAGE_CLASS_HEADER_KEY: KSS_HEADER_PREFIX + 'storage-class',
             MFA_HEADER_KEY: KSS_HEADER_PREFIX + 'mfa',
             RESTORE_HEADER_KEY: KSS_HEADER_PREFIX + 'restore',
+            TAGGING_COUNT_HEADER_KEY: KSS_HEADER_PREFIX + 'tagging-count',
+            OBJECT_TYPE_HEADER_KEY: KSS_HEADER_PREFIX + 'object-type',
+            APPEND_NEXT_POSITION_HEADER_KEY: KSS_HEADER_PREFIX + 'append-next-position'
         },
         'aws': {
             HEADER_PREFIX_KEY: AWS_HEADER_PREFIX,
@@ -136,6 +142,7 @@ class Provider(object):
             STORAGE_CLASS_HEADER_KEY: AWS_HEADER_PREFIX + 'storage-class',
             MFA_HEADER_KEY: AWS_HEADER_PREFIX + 'mfa',
             RESTORE_HEADER_KEY: AWS_HEADER_PREFIX + 'restore',
+            TAGGING_COUNT_HEADER_KEY: AWS_HEADER_PREFIX + 'tagging-count'
         },
         'google': {
             HEADER_PREFIX_KEY: GOOG_HEADER_PREFIX,
@@ -440,6 +447,10 @@ class Provider(object):
         self.version_id = header_info_map[VERSION_ID_HEADER_KEY]
         self.mfa_header = header_info_map[MFA_HEADER_KEY]
         self.restore_header = header_info_map[RESTORE_HEADER_KEY]
+        self.tagging_count_header = header_info_map[TAGGING_COUNT_HEADER_KEY]
+        self.object_type_header = header_info_map[OBJECT_TYPE_HEADER_KEY]
+        self.append_next_position_header = header_info_map[APPEND_NEXT_POSITION_HEADER_KEY]
+
 
     def configure_errors(self):
         error_map = self.ErrorMap[self.name]
